"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodesController = void 0;
const express_1 = __importDefault(require("express"));
const config_1 = __importDefault(require("../config"));
const LoadNodesAndCredentials_1 = require("../LoadNodesAndCredentials");
const ResponseHelper = __importStar(require("../ResponseHelper"));
const helpers_1 = require("../CommunityNodes/helpers");
const packageModel_1 = require("../CommunityNodes/packageModel");
const constants_1 = require("../constants");
const UserManagementHelper_1 = require("../UserManagement/UserManagementHelper");
const push_1 = require("../push");
const typedi_1 = require("typedi");
const InternalHooks_1 = require("../InternalHooks");
const { PACKAGE_NOT_INSTALLED, PACKAGE_NAME_NOT_PROVIDED } = constants_1.RESPONSE_ERROR_MESSAGES;
exports.nodesController = express_1.default.Router();
exports.nodesController.use((req, res, next) => {
    if (!(0, UserManagementHelper_1.isAuthenticatedRequest)(req) || req.user.globalRole.name !== 'owner') {
        res.status(403).json({ status: 'error', message: 'Unauthorized' });
        return;
    }
    next();
});
exports.nodesController.use((req, res, next) => {
    if (config_1.default.getEnv('executions.mode') === 'queue' && req.method !== 'GET') {
        res.status(400).json({
            status: 'error',
            message: 'Package management is disabled when running in "queue" mode',
        });
        return;
    }
    next();
});
exports.nodesController.post('/', ResponseHelper.send(async (req) => {
    const { name } = req.body;
    if (!name) {
        throw new ResponseHelper.BadRequestError(PACKAGE_NAME_NOT_PROVIDED);
    }
    let parsed;
    try {
        parsed = (0, helpers_1.parseNpmPackageName)(name);
    }
    catch (error) {
        throw new ResponseHelper.BadRequestError(error instanceof Error ? error.message : 'Failed to parse package name');
    }
    if (parsed.packageName === constants_1.STARTER_TEMPLATE_NAME) {
        throw new ResponseHelper.BadRequestError([
            `Package "${parsed.packageName}" is only a template`,
            'Please enter an actual package to install',
        ].join('.'));
    }
    const isInstalled = await (0, packageModel_1.isPackageInstalled)(parsed.packageName);
    const hasLoaded = (0, helpers_1.hasPackageLoaded)(name);
    if (isInstalled && hasLoaded) {
        throw new ResponseHelper.BadRequestError([
            `Package "${parsed.packageName}" is already installed`,
            'To update it, click the corresponding button in the UI',
        ].join('.'));
    }
    const packageStatus = await (0, helpers_1.checkNpmPackageStatus)(name);
    if (packageStatus.status !== 'OK') {
        throw new ResponseHelper.BadRequestError(`Package "${name}" is banned so it cannot be installed`);
    }
    let installedPackage;
    try {
        installedPackage = await typedi_1.Container.get(LoadNodesAndCredentials_1.LoadNodesAndCredentials).loadNpmModule(parsed.packageName, parsed.version);
    }
    catch (error) {
        const errorMessage = error instanceof Error ? error.message : constants_1.UNKNOWN_FAILURE_REASON;
        void typedi_1.Container.get(InternalHooks_1.InternalHooks).onCommunityPackageInstallFinished({
            user: req.user,
            input_string: name,
            package_name: parsed.packageName,
            success: false,
            package_version: parsed.version,
            failure_reason: errorMessage,
        });
        const message = [`Error loading package "${name}"`, errorMessage].join(':');
        const clientError = error instanceof Error ? (0, helpers_1.isClientError)(error) : false;
        throw new ResponseHelper[clientError ? 'BadRequestError' : 'InternalServerError'](message);
    }
    if (!hasLoaded)
        (0, helpers_1.removePackageFromMissingList)(name);
    const pushInstance = typedi_1.Container.get(push_1.Push);
    installedPackage.installedNodes.forEach((node) => {
        pushInstance.send('reloadNodeType', {
            name: node.type,
            version: node.latestVersion,
        });
    });
    void typedi_1.Container.get(InternalHooks_1.InternalHooks).onCommunityPackageInstallFinished({
        user: req.user,
        input_string: name,
        package_name: parsed.packageName,
        success: true,
        package_version: parsed.version,
        package_node_names: installedPackage.installedNodes.map((node) => node.name),
        package_author: installedPackage.authorName,
        package_author_email: installedPackage.authorEmail,
    });
    return installedPackage;
}));
exports.nodesController.get('/', ResponseHelper.send(async () => {
    const installedPackages = await (0, packageModel_1.getAllInstalledPackages)();
    if (installedPackages.length === 0)
        return [];
    let pendingUpdates;
    try {
        const command = ['npm', 'outdated', '--json'].join(' ');
        await (0, helpers_1.executeCommand)(command, { doNotHandleError: true });
    }
    catch (error) {
        if ((0, helpers_1.isNpmError)(error) && error.code === 1) {
            pendingUpdates = JSON.parse(error.stdout);
        }
    }
    let hydratedPackages = (0, helpers_1.matchPackagesWithUpdates)(installedPackages, pendingUpdates);
    try {
        const missingPackages = config_1.default.get('nodes.packagesMissing');
        if (missingPackages) {
            hydratedPackages = (0, helpers_1.matchMissingPackages)(hydratedPackages, missingPackages);
        }
    }
    catch (_) {
    }
    return hydratedPackages;
}));
exports.nodesController.delete('/', ResponseHelper.send(async (req) => {
    const { name } = req.query;
    if (!name) {
        throw new ResponseHelper.BadRequestError(PACKAGE_NAME_NOT_PROVIDED);
    }
    try {
        (0, helpers_1.sanitizeNpmPackageName)(name);
    }
    catch (error) {
        const message = error instanceof Error ? error.message : constants_1.UNKNOWN_FAILURE_REASON;
        throw new ResponseHelper.BadRequestError(message);
    }
    const installedPackage = await (0, packageModel_1.findInstalledPackage)(name);
    if (!installedPackage) {
        throw new ResponseHelper.BadRequestError(PACKAGE_NOT_INSTALLED);
    }
    try {
        await typedi_1.Container.get(LoadNodesAndCredentials_1.LoadNodesAndCredentials).removeNpmModule(name, installedPackage);
    }
    catch (error) {
        const message = [
            `Error removing package "${name}"`,
            error instanceof Error ? error.message : constants_1.UNKNOWN_FAILURE_REASON,
        ].join(':');
        throw new ResponseHelper.InternalServerError(message);
    }
    const pushInstance = typedi_1.Container.get(push_1.Push);
    installedPackage.installedNodes.forEach((node) => {
        pushInstance.send('removeNodeType', {
            name: node.type,
            version: node.latestVersion,
        });
    });
    void typedi_1.Container.get(InternalHooks_1.InternalHooks).onCommunityPackageDeleteFinished({
        user: req.user,
        package_name: name,
        package_version: installedPackage.installedVersion,
        package_node_names: installedPackage.installedNodes.map((node) => node.name),
        package_author: installedPackage.authorName,
        package_author_email: installedPackage.authorEmail,
    });
}));
exports.nodesController.patch('/', ResponseHelper.send(async (req) => {
    const { name } = req.body;
    if (!name) {
        throw new ResponseHelper.BadRequestError(PACKAGE_NAME_NOT_PROVIDED);
    }
    const previouslyInstalledPackage = await (0, packageModel_1.findInstalledPackage)(name);
    if (!previouslyInstalledPackage) {
        throw new ResponseHelper.BadRequestError(PACKAGE_NOT_INSTALLED);
    }
    try {
        const newInstalledPackage = await typedi_1.Container.get(LoadNodesAndCredentials_1.LoadNodesAndCredentials).updateNpmModule((0, helpers_1.parseNpmPackageName)(name).packageName, previouslyInstalledPackage);
        const pushInstance = typedi_1.Container.get(push_1.Push);
        previouslyInstalledPackage.installedNodes.forEach((node) => {
            pushInstance.send('removeNodeType', {
                name: node.type,
                version: node.latestVersion,
            });
        });
        newInstalledPackage.installedNodes.forEach((node) => {
            pushInstance.send('reloadNodeType', {
                name: node.name,
                version: node.latestVersion,
            });
        });
        void typedi_1.Container.get(InternalHooks_1.InternalHooks).onCommunityPackageUpdateFinished({
            user: req.user,
            package_name: name,
            package_version_current: previouslyInstalledPackage.installedVersion,
            package_version_new: newInstalledPackage.installedVersion,
            package_node_names: newInstalledPackage.installedNodes.map((node) => node.name),
            package_author: newInstalledPackage.authorName,
            package_author_email: newInstalledPackage.authorEmail,
        });
        return newInstalledPackage;
    }
    catch (error) {
        previouslyInstalledPackage.installedNodes.forEach((node) => {
            const pushInstance = typedi_1.Container.get(push_1.Push);
            pushInstance.send('removeNodeType', {
                name: node.type,
                version: node.latestVersion,
            });
        });
        const message = [
            `Error removing package "${name}"`,
            error instanceof Error ? error.message : constants_1.UNKNOWN_FAILURE_REASON,
        ].join(':');
        throw new ResponseHelper.InternalServerError(message);
    }
}));
//# sourceMappingURL=nodes.api.js.map