"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tagsController = void 0;
const express_1 = __importDefault(require("express"));
const Db = __importStar(require("../Db"));
const ExternalHooks_1 = require("../ExternalHooks");
const ResponseHelper = __importStar(require("../ResponseHelper"));
const config_1 = __importDefault(require("../config"));
const TagHelpers = __importStar(require("../TagHelpers"));
const GenericHelpers_1 = require("../GenericHelpers");
const TagEntity_1 = require("../databases/entities/TagEntity");
const typedi_1 = require("typedi");
exports.tagsController = express_1.default.Router();
const workflowsEnabledMiddleware = (req, res, next) => {
    if (config_1.default.getEnv('workflowTagsDisabled')) {
        throw new ResponseHelper.BadRequestError('Workflow tags are disabled');
    }
    next();
};
exports.tagsController.get('/', workflowsEnabledMiddleware, ResponseHelper.send(async (req) => {
    if (req.query.withUsageCount === 'true') {
        const tablePrefix = config_1.default.getEnv('database.tablePrefix');
        return TagHelpers.getTagsWithCountDb(tablePrefix);
    }
    return Db.collections.Tag.find({ select: ['id', 'name', 'createdAt', 'updatedAt'] });
}));
exports.tagsController.post('/', workflowsEnabledMiddleware, ResponseHelper.send(async (req) => {
    const newTag = new TagEntity_1.TagEntity();
    newTag.name = req.body.name.trim();
    await typedi_1.Container.get(ExternalHooks_1.ExternalHooks).run('tag.beforeCreate', [newTag]);
    await (0, GenericHelpers_1.validateEntity)(newTag);
    const tag = await Db.collections.Tag.save(newTag);
    await typedi_1.Container.get(ExternalHooks_1.ExternalHooks).run('tag.afterCreate', [tag]);
    return tag;
}));
exports.tagsController.patch('/:id(\\d+)', workflowsEnabledMiddleware, ResponseHelper.send(async (req) => {
    const { name } = req.body;
    const { id } = req.params;
    const newTag = new TagEntity_1.TagEntity();
    newTag.id = id;
    newTag.name = name.trim();
    await typedi_1.Container.get(ExternalHooks_1.ExternalHooks).run('tag.beforeUpdate', [newTag]);
    await (0, GenericHelpers_1.validateEntity)(newTag);
    const tag = await Db.collections.Tag.save(newTag);
    await typedi_1.Container.get(ExternalHooks_1.ExternalHooks).run('tag.afterUpdate', [tag]);
    return tag;
}));
exports.tagsController.delete('/:id(\\d+)', workflowsEnabledMiddleware, ResponseHelper.send(async (req) => {
    if (config_1.default.getEnv('userManagement.isInstanceOwnerSetUp') === true &&
        req.user.globalRole.name !== 'owner') {
        throw new ResponseHelper.UnauthorizedError('You are not allowed to perform this action', 'Only owners can remove tags');
    }
    const id = req.params.id;
    await typedi_1.Container.get(ExternalHooks_1.ExternalHooks).run('tag.beforeDelete', [id]);
    await Db.collections.Tag.delete({ id });
    await typedi_1.Container.get(ExternalHooks_1.ExternalHooks).run('tag.afterDelete', [id]);
    return true;
}));
//# sourceMappingURL=tags.api.js.map