"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.samlControllerProtected = void 0;
const express_1 = __importDefault(require("express"));
const samlEnabledMiddleware_1 = require("../middleware/samlEnabledMiddleware");
const saml_service_ee_1 = require("../saml.service.ee");
const constants_1 = require("../constants");
const ResponseHelper_1 = require("../../../ResponseHelper");
const jwt_1 = require("../../../auth/jwt");
const samlHelpers_1 = require("../samlHelpers");
exports.samlControllerProtected = express_1.default.Router();
exports.samlControllerProtected.get(constants_1.SamlUrls.config, samlEnabledMiddleware_1.samlLicensedOwnerMiddleware, (req, res) => {
    const prefs = saml_service_ee_1.SamlService.getInstance().getSamlPreferences();
    return res.send(prefs);
});
exports.samlControllerProtected.post(constants_1.SamlUrls.config, samlEnabledMiddleware_1.samlLicensedOwnerMiddleware, async (req, res) => {
    if ((0, samlHelpers_1.isSamlPreferences)(req.body)) {
        const result = await saml_service_ee_1.SamlService.getInstance().setSamlPreferences(req.body);
        return res.send(result);
    }
    else {
        throw new ResponseHelper_1.BadRequestError('Body is not a SamlPreferences object');
    }
});
exports.samlControllerProtected.get(constants_1.SamlUrls.acs, samlEnabledMiddleware_1.samlLicensedAndEnabledMiddleware, async (req, res) => {
    const loginResult = await saml_service_ee_1.SamlService.getInstance().handleSamlLogin(req, 'redirect');
    if (loginResult) {
        if (loginResult.authenticatedUser) {
            await (0, jwt_1.issueCookie)(res, loginResult.authenticatedUser);
            if (loginResult.onboardingRequired) {
                return res.redirect(constants_1.SamlUrls.samlOnboarding);
            }
            else {
                return res.redirect(constants_1.SamlUrls.defaultRedirect);
            }
        }
    }
    throw new ResponseHelper_1.AuthError('SAML Authentication failed');
});
exports.samlControllerProtected.post(constants_1.SamlUrls.acs, samlEnabledMiddleware_1.samlLicensedAndEnabledMiddleware, async (req, res) => {
    const loginResult = await saml_service_ee_1.SamlService.getInstance().handleSamlLogin(req, 'post');
    if (loginResult) {
        if (loginResult.authenticatedUser) {
            await (0, jwt_1.issueCookie)(res, loginResult.authenticatedUser);
            if (loginResult.onboardingRequired) {
                return res.redirect(constants_1.SamlUrls.samlOnboarding);
            }
            else {
                return res.redirect(constants_1.SamlUrls.defaultRedirect);
            }
        }
    }
    throw new ResponseHelper_1.AuthError('SAML Authentication failed');
});
exports.samlControllerProtected.get(constants_1.SamlUrls.initSSO, samlEnabledMiddleware_1.samlLicensedAndEnabledMiddleware, async (req, res) => {
    const url = saml_service_ee_1.SamlService.getInstance().getRedirectLoginRequestUrl();
    if (url) {
        return res.status(301).send(url);
    }
    else {
        throw new ResponseHelper_1.AuthError('SAML redirect failed, please check your SAML configuration.');
    }
});
//# sourceMappingURL=saml.controller.protected.ee.js.map