"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SamlService = void 0;
const Db = __importStar(require("../../Db"));
const n8n_workflow_1 = require("n8n-workflow");
const ResponseHelper_1 = require("../../ResponseHelper");
const serviceProvider_ee_1 = require("./serviceProvider.ee");
const ssoHelpers_1 = require("../ssoHelpers");
const constants_1 = require("./constants");
const samlify_1 = require("samlify");
const samlHelpers_1 = require("./samlHelpers");
class SamlService {
    get attributeMapping() {
        return this._attributeMapping;
    }
    set attributeMapping(mapping) {
        this._attributeMapping = mapping;
    }
    get metadata() {
        return this._metadata;
    }
    set metadata(metadata) {
        this._metadata = metadata;
    }
    constructor() {
        this._attributeMapping = {
            email: 'http://schemas.xmlsoap.org/ws/2005/05/identity/claims/emailaddress',
            firstName: 'http://schemas.xmlsoap.org/ws/2005/05/identity/claims/firstname',
            lastName: 'http://schemas.xmlsoap.org/ws/2005/05/identity/claims/lastname',
            userPrincipalName: 'http://schemas.xmlsoap.org/ws/2005/05/identity/claims/upn',
        };
        this._metadata = '';
        this.loadSamlPreferences()
            .then(() => {
            n8n_workflow_1.LoggerProxy.debug('Initializing SAML service');
        })
            .catch(() => {
            n8n_workflow_1.LoggerProxy.error('Error initializing SAML service');
        });
    }
    static getInstance() {
        if (!SamlService.instance) {
            SamlService.instance = new SamlService();
        }
        return SamlService.instance;
    }
    async init() {
        await this.loadSamlPreferences();
    }
    getIdentityProviderInstance(forceRecreate = false) {
        if (this.identityProviderInstance === undefined || forceRecreate) {
            this.identityProviderInstance = (0, samlify_1.IdentityProvider)({
                metadata: this.metadata,
            });
        }
        return this.identityProviderInstance;
    }
    getRedirectLoginRequestUrl() {
        const loginRequest = (0, serviceProvider_ee_1.getServiceProviderInstance)().createLoginRequest(this.getIdentityProviderInstance(), 'redirect');
        n8n_workflow_1.LoggerProxy.debug(loginRequest.context);
        return loginRequest.context;
    }
    async handleSamlLogin(req, binding) {
        const attributes = await this.getAttributesFromLoginResponse(req, binding);
        if (attributes.email) {
            const user = await Db.collections.User.findOne({
                where: { email: attributes.email },
                relations: ['globalRole', 'authIdentities'],
            });
            if (user) {
                if (user.authIdentities.find((e) => e.providerType === 'saml' && e.providerId === attributes.userPrincipalName)) {
                    return {
                        authenticatedUser: user,
                        attributes,
                        onboardingRequired: false,
                    };
                }
                else {
                    const updatedUser = await (0, samlHelpers_1.updateUserFromSamlAttributes)(user, attributes);
                    return {
                        authenticatedUser: updatedUser,
                        attributes,
                        onboardingRequired: true,
                    };
                }
            }
            else {
                if ((0, ssoHelpers_1.isSsoJustInTimeProvisioningEnabled)()) {
                    const newUser = await (0, samlHelpers_1.createUserFromSamlAttributes)(attributes);
                    return {
                        authenticatedUser: newUser,
                        attributes,
                        onboardingRequired: true,
                    };
                }
            }
        }
        return undefined;
    }
    getSamlPreferences() {
        return {
            mapping: this.attributeMapping,
            metadata: this.metadata,
            loginEnabled: (0, samlHelpers_1.isSamlLoginEnabled)(),
            loginLabel: (0, samlHelpers_1.getSamlLoginLabel)(),
        };
    }
    async setSamlPreferences(prefs) {
        this.attributeMapping = prefs.mapping;
        this.metadata = prefs.metadata;
        (0, samlHelpers_1.setSamlLoginEnabled)(prefs.loginEnabled);
        (0, samlHelpers_1.setSamlLoginLabel)(prefs.loginLabel);
        this.getIdentityProviderInstance(true);
        await this.saveSamlPreferences();
    }
    async loadSamlPreferences() {
        const samlPreferences = await Db.collections.Settings.findOne({
            where: { key: constants_1.SAML_PREFERENCES_DB_KEY },
        });
        if (samlPreferences) {
            const prefs = (0, n8n_workflow_1.jsonParse)(samlPreferences.value);
            if (prefs) {
                await this.setSamlPreferences(prefs);
                return prefs;
            }
        }
        return;
    }
    async saveSamlPreferences() {
        const samlPreferences = await Db.collections.Settings.findOne({
            where: { key: constants_1.SAML_PREFERENCES_DB_KEY },
        });
        const settingsValue = JSON.stringify(this.getSamlPreferences());
        if (samlPreferences) {
            samlPreferences.value = settingsValue;
            await Db.collections.Settings.save(samlPreferences);
        }
        else {
            await Db.collections.Settings.save({
                key: constants_1.SAML_PREFERENCES_DB_KEY,
                value: settingsValue,
                loadOnStartup: true,
            });
        }
    }
    async getAttributesFromLoginResponse(req, binding) {
        let parsedSamlResponse;
        try {
            parsedSamlResponse = await (0, serviceProvider_ee_1.getServiceProviderInstance)().parseLoginResponse(this.getIdentityProviderInstance(), binding, req);
        }
        catch (error) {
            throw error;
        }
        const { attributes, missingAttributes } = (0, samlHelpers_1.getMappedSamlAttributesFromFlowResult)(parsedSamlResponse, this.attributeMapping);
        if (!attributes) {
            throw new ResponseHelper_1.AuthError('SAML Authentication failed. Invalid SAML response.');
        }
        if (!attributes.email && missingAttributes.length > 0) {
            throw new ResponseHelper_1.AuthError(`SAML Authentication failed. Invalid SAML response (missing attributes: ${missingAttributes.join(', ')}).`);
        }
        return attributes;
    }
}
exports.SamlService = SamlService;
//# sourceMappingURL=saml.service.ee.js.map